/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#include "FileReader.h"

#define FILEREADER_DEBUG 0
#define FILEREADER_TRACE 1

//-----------------------------------------------------------------------------
//	Ctor should only init the struct. Allocations are in Start().
//-----------------------------------------------------------------------------
FileReader::FileReader()
	: _pIFileMgr(NULL)
	, _pIFile(NULL)
	, _pReturnBuffer(NULL)
	, _pReadBuffer(NULL)
	, _idxFilePos(0)
	, _idxReadBufPos(0)
	, _nReadBufLen(0)
{
}

//-----------------------------------------------------------------------------
bool FileReader::Start(int lenReturnBuffer)
{
	if (_pIFileMgr)
	{
		DEBUG(( "FileReader.Start() WARNING: already started." ));
		return true;
	}

	ASSERT(GETSHELL());
	if (ISHELL_CreateInstance(GETSHELL(), AEECLSID_FILEMGR, (void **)&_pIFileMgr) != SUCCESS)
	{
		return false;
	}

	if (!_pReturnBuffer)
	{
		_pReturnBuffer = (char *)MEMALLOC(lenReturnBuffer);
	}
	if (!_pReadBuffer)
	{
		_pReadBuffer = (char*)MEMALLOC(LEN_OF_READ_BUFFER);
	}
	ASSERT(_pReturnBuffer);
	ASSERT(_pReadBuffer);
	if (!_pReturnBuffer || !_pReadBuffer)
	{
		DEBUG(( "FileReader.Start() ERROR: could not allocate buffers." ));
		Stop();
		return false;
	}
	return true;
}
//-----------------------------------------------------------------------------
void FileReader::Stop()
{
	Close();

	if (_pIFileMgr)
	{
		if (IFILEMGR_Release(_pIFileMgr) > 0)
		{
			DEBUG(( "FileReader.Stop() WARNING: _pIFileMgr is still referenced." ));
		}
		_pIFileMgr = NULL;
	}
	MEMFREE(_pReadBuffer); _pReadBuffer = NULL;
	MEMFREE(_pReturnBuffer); _pReturnBuffer = NULL;
}
//-----------------------------------------------------------------------------
bool FileReader::Open(const char* pszFilename, OpenFileMode ofm)
{
	TRACE(( "FileReader::Open('%s',%s)", pszFilename, (ofm==_OFM_READ ? "READ" : "WRITE") ));

	_idxFilePos = 0;
	_idxReadBufPos = 0;
	_nReadBufLen = 0;

	if (_pIFile)
	{
		DEBUG(( "FileReader.Open() WARNING: another file was still open." ));
		Close();
	}
	if ((_pIFile = IFILEMGR_OpenFile(_pIFileMgr, pszFilename, ofm)) == NULL)
	{
		ASSERT(_pIFile);
		DEBUG(( "FileReader.Open() ERROR: could not open '%s'.", pszFilename ));
		return false;
	}
	return true;
}
//-----------------------------------------------------------------------------
void FileReader::Close()
{
	if(!_pIFile)
	{
		DEBUG(( "FileReader.Close() WARNING: file is already closed." ));
	}
	else if (IFILE_Release(_pIFile) > 0)
	{
		DEBUG(( "FileReader.Close() WARNING: file is still referenced." ));
	}
	_pIFile = NULL;
}

//-----------------------------------------------------------------------------
void FileReader::Rewind()
{
	TRACE(( "FileReader.Rewind()" ));

	ASSERT(_pIFileMgr);
	ASSERT(_pIFile);
	if (!_pIFileMgr) return;
	if (!_pIFile) return;

	_idxFilePos = 0;
	_idxReadBufPos = 0;
	_nReadBufLen = 0;

	IFILE_Seek(_pIFile, _SEEK_START, 0);
}

//-----------------------------------------------------------------------------
char* FileReader::ReadLine()
{
#if FILEREADER_TRACE
	TRACE(( "FileReader.ReadLine()" ));
#endif

	return ReadWord(NULL);
}

//-----------------------------------------------------------------------------
char* FileReader::ReadWord(const char* pDelimiters)
{
#if FILEREADER_TRACE
	if (pDelimiters) TRACE(( "FileReader.ReadWord()" ));
#endif

	ASSERT(_pIFileMgr);
	ASSERT(_pIFile);
	if (!_pIFileMgr) return false;
	if (!_pIFile) return false;

	//scan file for delimiters
	char ch;
	boolean isFound = false;
	int idx, idxWrite = 0;
	int nDelimiters = 0;
	if (pDelimiters)
		nDelimiters = STRLEN(pDelimiters);
	do
	{
		//Fill the read buf if it is empty or consumed
		if ((!_nReadBufLen) || (_nReadBufLen <= _idxReadBufPos))
		{
			_idxReadBufPos = 0;
			_nReadBufLen = IFILE_Read(_pIFile, _pReadBuffer, LEN_OF_READ_BUFFER);
		}
		if (!_nReadBufLen) //didn't read anything on this pass?
			break;

		ch = _pReadBuffer[_idxReadBufPos++];
		_idxFilePos++;

#if FILEREADER_DEBUG
		DEBUG(( "ch = 0x%x, %c, %i", ch,ch,ch ));
#endif
		if (ch == '\r') continue;	//skip carriage returns
		if (ch == '\n') break;		//stop at newline

		//Check ch against delimiters
		for (idx=nDelimiters-1; idx >= 0; idx--)
		{
			if (ch == pDelimiters[idx])
			{
				isFound = true;
				break;
			}
		}
		if (isFound) break; //don't return the delimiter

		_pReturnBuffer[idxWrite++] = ch;

		//This class won't handle lines longer than the ReturnBuffer. Truncate.
		if (idxWrite >= LEN_OF_RETURN_BUFFER-1)
		{
			break;
		}
	}
	while (true);

//DEBUG(( "found ch %c at %d", ch, _idxFilePos-1 ));

	_pReturnBuffer[idxWrite++] = '\0';
	if ((_pReturnBuffer[0] == '\0') && !_nReadBufLen)
	{
		return NULL;	//EndOfFile (as opposed to an empty line)
	}
	return _pReturnBuffer;
}

//-----------------------------------------------------------------------------
bool FileReader::WriteChar(const char ch)
{
	ASSERT(_pIFileMgr);
	ASSERT(_pIFile);
	if (!_pIFileMgr) return false;
	if (!_pIFile) return false;

	uint32 nBytes;

	//If the current position is in the middle of the read buffer,
	//we need to rewind the file pointer to the current position.
	if (_idxReadBufPos < _nReadBufLen)
	{
		nBytes = IFILE_Seek(_pIFile, _SEEK_START, _idxFilePos);
		if (nBytes == EFAILED)
		{
			DEBUG(( "FileReader::WriteChar() ERROR IFILE_Seek() failed pos=%d.", _idxFilePos ));
			return false;
		}
	}
	nBytes = IFILE_Write(_pIFile, &ch, (uint32)1);
	if (!nBytes)
	{
		DEBUG(( "FileReader::WriteChar() ERROR IFILE_Write() failed." ));
		return false;
	}

DEBUG(( "wrote ch %c (%d bytes) at %d", ch, nBytes, _idxFilePos ));
	//Apparently, it leaves the file position alone after a write
	//	//The file position gets messed up by writing, so just rewind
	//	//	//Advance the positions by one char to match the new file position.
	//	//	_idxReadBufPos++;
	//	//	_idxFilePos++;
	//Rewind();
	return true;
}

