/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#include "AMTPUtil.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Get the name of the main AMTP resource file.
 */
char * TBUTIL_GetResFile() 
{
	return const_cast<char *>("amtp.bar");
}


/**
 * Get the name of the ancilliary AMTP game-specific resource file.
 */
char * TBUTIL_GetGameResFile() 
{
	return const_cast<char *>("amtpgame.bar");
}



/**
 * Converts a version string to a version number in the format required by
 * the AMTP server.
 *
 * Params:
 *   versionStr - A string of the form major.minor.build. All three
 *                components must exist in the string (e.g. "1.0.4").
 *
 * Returns:
 *    An int32 containing the version number in the format required by the
 *    AMTP server. The major version is stored in the first 2 bytes, the
 *    minor version is stored in the next two bytes, and the build number
 *    is stored in the last 4 bytes.
 */
int32 TBUTIL_CreateVersionNumber(char *versionStr)
{
	int versionNum = 0x00000000;

	if (versionStr) {
		int idx = STRLEN(versionStr);
		int32 num;
		int32 factor;
        char ch;

		// Read build number and store in bytes 5-8
		num = 0;
		factor = 1;
		ch = versionStr[--idx];
		while (idx > 0 && ch != '.') {
			num += ((ch - '0') * factor);
			factor *= 10;
			ch = versionStr[--idx];
		}
		versionNum = num;

		// Read minor version and store in bytes 3-4
		num = 0;
		factor = 1;
		ch = versionStr[--idx];
		while (idx > 0 && ch != '.') {
			num += ((ch - '0') * factor);
			factor *= 10;
			ch = versionStr[--idx];
		}
		versionNum = versionNum | (num << 16);

		// Read major version and store in bytes 1-2
		num = 0;
		factor = 1;
		ch = versionStr[--idx];
		while (idx >= 0 && ch != '.') {
			num += ((ch - '0') * factor);
			factor *= 10;
			idx--;
			if (idx >= 0) {
				ch = versionStr[--idx];
			}
		}
		versionNum = versionNum | (num << 24);
	}

	return versionNum;
}


/**
 * Converts an 8-bit character to a 16-bit character.
 *
 * NOTE: This function could stand some optimization.
 *
 * Params:
 *   in_Char - Input 8-byte character.
 *
 * Returns:
 *    Converted wide 16-byte character.
 */
AECHAR TBUTIL_CharToWchar( const char in_Char )
{
	char	a_SrcBuf[ 2 ];
	AECHAR	a_DstBuf[ 2 ];

	a_SrcBuf[ 0 ] = in_Char;
	a_SrcBuf[ 1 ] = 0;
	STRTOWSTR( a_SrcBuf, a_DstBuf, sizeof( a_DstBuf ) );
	return a_DstBuf[ 0 ];
}


/**
 * Converts an 8-bit character string to a 16-bit character string.
 *
 * Patches a bug that appears in the Brew string handling for Nokia 3589i.
 * The runtime code for this phone treats a newline at the begining of a
 * line (beginning of string or right after another newline) as a string
 * terminator.  The workaround applied here is to insert an extra space
 * character before every newline character.  This seems a safe for all
 * phones wherever STRTOWSTR would be used but we could test whether this
 * is a 3589i and divert to STRTOWSTR otherwise.  This function may also
 * differ from STRTOWSTR in that it will fill and terminate the output
 * buffer up to in_BufSize and still return out_pBuf without complaint
 * if the buffer was not big enough.
 */
AECHAR * TBUTIL_StrToWstr( const char *in_pSz, AECHAR *out_pBuf, int out_BufSize )
{
	if (Platform_GetPhoneID() == PLATFORM_ID_3589I ||
		Platform_GetPhoneID() == PLATFORM_ID_3589I_EM) 
	{
		const char	*a_pCopySrc = in_pSz;
		AECHAR		*a_pCopyDst = out_pBuf;
		AECHAR		*a_pDstBufEndPos = & out_pBuf[ ( out_BufSize / 2 ) - 1 ];

		// NOTE: a_pDstBufEndPos is the last possible wide char pos of
		// the buffer must hold the terminator buffer is completely
		// full.

		for ( ;; ){
			for ( ; a_pCopyDst < a_pDstBufEndPos; ){
				if ( *a_pCopySrc == '\n' ) break;
				if ( *a_pCopySrc == 0 ) break;
				*a_pCopyDst = TBUTIL_CharToWchar( *a_pCopySrc );
				a_pCopySrc++;
				a_pCopyDst++;
			}

			if( a_pCopyDst >= a_pDstBufEndPos ) break;

			if ( *a_pCopySrc == '\n' ){
				*a_pCopyDst = TBUTIL_CharToWchar( ' ' );
				// a_pCopySrc is not advanced here.
				a_pCopyDst++;
				if( a_pCopyDst >= a_pDstBufEndPos ) break;
				*a_pCopyDst = TBUTIL_CharToWchar( '\n' );
				a_pCopySrc++;
				a_pCopyDst++;
			}else{
				ASSERT( *a_pCopySrc == 0 );
				break;
			}
		}
		ASSERT( a_pCopyDst <= a_pDstBufEndPos );

		// Write the terminator.
		*a_pCopyDst = TBUTIL_CharToWchar( 0 );

		return ( out_BufSize > 0 ? out_pBuf : NULL );
	}
	else {
		return STRTOWSTR(in_pSz, out_pBuf, out_BufSize);
	}
}


/**
 * Load a resource string into a single-byte character buffer. The returned buffer
 * must be freed by the caller using MEMFREE.
 *
 * Params:
 *   rsrcFile	- Name of the resource file containing resource
 *   rsrcID		- Resource ID of string to load
 */
char * TBUTIL_LoadResStr(char *rsrcFile, int rsrcID)
{
	return TBUTIL_LoadResStrNL(rsrcFile, rsrcID, false, ' ');
}


/**
 * Load a resource string into a double-byte AECHAR. The returned buffer must be 
 * freed by the caller using MEMFREE.
 *
 * Params:
 *   rsrcFile	- Name of the resource file containing resource
 *   rsrcID		- Resource ID of string to load
 */
AECHAR * TBUTIL_LoadResWStr(char *rsrcFile, int rsrcID)
{
	return TBUTIL_LoadResWStrNL(rsrcFile, rsrcID, false, ' ');
}


/**
 * Load a resource string into a single byte character buffer with optional new-line
 * character insertion. The returned buffer must be freed by the caller using
 * MEMFREE.
 *
 * Params:
 *   rsrcFile        - Name of the resource file containing resource
 *   rsrcID          - Resource ID of string to load
 *   insertNewLines  - Indicated whether new lines should be inserted into the string
 *   replacementChar - Character that will be replaced with new lines. If insertNewLines
 *                     is false, this parameter is ignored.
 */
char * TBUTIL_LoadResStrNL(char *rsrcFile, int rsrcID, bool insertNewLines, char replacementChar)
{
	AECHAR *rsrcBuf = TBUTIL_LoadResWStrNL(rsrcFile, rsrcID, insertNewLines, replacementChar);

	int numChars = WSTRLEN(rsrcBuf);
	char * buf = (char *)MEMALLOC(numChars + 1);
	WSTRTOSTR(rsrcBuf, buf, numChars + 1);
	
	MEMFREE(rsrcBuf);

	return buf;
}


/**
 * Load a resource string into a double-byte AECHAR buffer with optional new-line
 * character insertion. The returned buffer must be freed by the caller using
 * MEMFREE.
 *
 * Params:
 *   rsrcFile        - Name of the resource file containing resource
 *   rsrcID          - Resource ID of string to load
 *   insertNewLines  - Indicated whether new lines should be inserted into the string
 *   replacementChar - Character that will be replaced with new lines. If insertNewLines
 *                     is false, this parameter is ignored.
 */
AECHAR * TBUTIL_LoadResWStrNL(char *rsrcFile, int rsrcID, bool insertNewLines, char replacementChar)
{
	AECHAR *rsrcBuf;
	uint32 numChars = 0;

	// This is supposed to give size in bytes, but actually gives the number of AECHARs
	ISHELL_LoadResDataEx(GETSHELL(), rsrcFile, rsrcID, RESTYPE_STRING, (void *)-1, &numChars);

	if (numChars == 0) {
		return NULL;
	}

	rsrcBuf = (AECHAR *)MEMALLOC((numChars + 1) * sizeof(AECHAR));
	ISHELL_LoadResString(GETSHELL(), rsrcFile, rsrcID, (AECHAR *)rsrcBuf, numChars);
	
	if (insertNewLines) {
		TBUTIL_InsertNewLines(rsrcBuf, replacementChar);
	}

	return rsrcBuf;
}


/**
 * Replaces all instances of the specified character with new-line characters.
 * Useful for adding new line characters to string resources.
 */
void TBUTIL_InsertNewLines(AECHAR *buffer, char replaceChar) {
	for (int i = (WSTRLEN(buffer) - 1); i >= 0; i--) {
		if (buffer[i] == replaceChar) {
			buffer[i] = '\n';
		}
	}
}


/**
 * Convert raw device-independent image data into a device-dependent bitmap.
 */
bool TBUTIL_CreateBitmap(ConvertedBitmapInfo *cbi, void *rawData, int rawDataSize)
{
	if (rawData && rawDataSize > 0) {
		void * bitmap = CONVERTBMP((void *)rawData, &cbi->bitmapInfo, &cbi->memAlloc);
		if (!bitmap) {
			return false;
		}
		if (bitmap && !cbi->memAlloc) {
			cbi->bitmap = MEMALLOC(rawDataSize);
			MEMCPY(cbi->bitmap, bitmap, rawDataSize);
		}
		else {
			cbi->bitmap = bitmap;
		}
		return true;
	}

	return false;
}


/**
 * Release a device-dependent bitmap created with AMTPUTIL_CreateBitmap.
 */
void TBUTIL_ReleaseBitmap(ConvertedBitmapInfo *cbi)
{
	if (cbi && cbi->bitmap) {
		if (cbi->memAlloc) {
			SYSFREE(cbi->bitmap);
		}
		else {
			FREE(cbi->bitmap);
		}
		cbi->bitmap = NULL;
		cbi->memAlloc = FALSE;
	}
}

#ifdef __cplusplus
} //extern "C"
#endif
