//-----------------------------------------------------------------------------
//	DBGLOG
//-----------------------------------------------------------------------------
#ifndef DBGLOG_H
#define DBGLOG_H
#include "Debug.h"	//DBGPRINTF
#include <stdarg.h>

#if DBGSTK_ENABLED
	struct DbgStk
	{
		enum StateEnum {
			STATE_NeedInit,
			STATE_NeedReport,
			STATE_Idle,
		};

		enum {
			DEPTH_TIMID		= 0x0100,
			DEPTH_BRAVE		= 0x1000,
			SIG_32			= 0x01020304,
		};

		void		*m_pProbeLo;
		void		*m_pProbeHi;
		void		*m_pProbeStung;
		StateEnum	 m_State;

		//////////////////////
		// Ctor and init.
		//////////////////////

		void InitProbes( const void *in_pVisitedAddr );

		DbgStk()
			: m_pProbeLo( NULL )
			, m_pProbeHi( NULL )
			, m_pProbeStung( NULL )
			, m_State( STATE_NeedInit )
		{
			InitProbes( GetSampleGrowthPtr() );
		}

		//////////////////////
		// Update and probe.
		//////////////////////

		void	 Update( void );
		void	 WriteSignature( void );
		void	 PerformDeepTest( void );
		void	*GetSampleGrowthPtr( void );
		void	 RecordVisitedAddr( const void *in_pVisitedAddr );
		

		////////////////
		// Accessors.
		////////////////

		void	*GetProbeLo( void ){	return m_pProbeLo; }
		void	*GetProbeHi( void ){	return m_pProbeHi; }
		void	*GetProbeStung( void ){	return m_pProbeStung; }
	};
#endif // DBGSTK_ENABLED


#if DBGLOG_ENABLED
	#include "AEEFile.h"	// For writing log to a file.


	struct DbgLog
	{
		enum {
			FLAG_WRITE_IMMEDIATE	= 0x01,
			FLAG_FLUSH_NEXT_WRITE	= 0x02,
			FLAG_AUTO_FLUSH			= 0x04,
			FLAG_FLUSH_DEEP			= 0x08,
			FLAG_ACTIVE				= 0x10,
			//
			BUFSIZE					= 128, // 1024,
			WRITE_FAILURE_CT_MAX	= 255,
		};

		char		 m_pBuf[ BUFSIZE ];
		//
		IFileMgr	*m_pFileMgr;
		IFile		*m_pFileHdl;
		char		*m_pBufTail; // Byte after last append, start of next append.
		char		*m_pBufWriteThreshhold;
		//
		char		*m_pFilePath;
		uint8		 m_Flags;
		uint8		 m_iWritesPerAutoFlush;
		uint8		 m_iAutoFlushCountdown;
		uint8		 m_iWriteFailureCt;

		// Inline Accessors.
		void	EnableFlags( uint8 in_Flags ){	m_Flags |= in_Flags;	}
		void	DisableFlags( uint8 in_Flags ){	m_Flags &= ~in_Flags;	}
		bool	CheckFlags( uint8 in_Flags ){	return (m_Flags & in_Flags) == in_Flags;	} 

		void	EnableWriteImmediate( void ){	EnableFlags( FLAG_WRITE_IMMEDIATE );		}
		void	DisableWriteImmediate( void ){	DisableFlags( FLAG_WRITE_IMMEDIATE );		}
		bool	CheckWriteImmediate( void ){	return CheckFlags( FLAG_WRITE_IMMEDIATE );	}

		void	EnableFlushNextWrite( void ){	EnableFlags( FLAG_FLUSH_NEXT_WRITE );		}
		void	DisableFlushNextWrite( void ){	DisableFlags( FLAG_FLUSH_NEXT_WRITE );		}
		bool	CheckFlushNextWrite( void ){	return CheckFlags( FLAG_FLUSH_NEXT_WRITE );	}

		void	EnableAutoFlush( void ){		EnableFlags( FLAG_AUTO_FLUSH );				}
		void	DisableAutoFlush( void ){		DisableFlags( FLAG_AUTO_FLUSH );			}
		bool	CheckAutoFlush( void ){			return CheckFlags( FLAG_AUTO_FLUSH );		}

		void	EnableFlushDeep( void ){		EnableFlags( FLAG_FLUSH_DEEP );				}
		void	DisableFlushDeep( void ){		DisableFlags( FLAG_FLUSH_DEEP );			}
		bool	CheckFlushDeep( void ){			return CheckFlags( FLAG_FLUSH_DEEP );		}

		// Stop() does everything that the destructor requires.
		void	Stop( void ){ DeepFlushNow(); }

		// Member Func prototypes.
		DbgLog();
		~DbgLog(){ Stop(); }

	public:
		void	Append_SimpStr( const char *in_pMsg );
		void	Append( char in_Char );
		void	Append( const AECHAR *in_pMsg );
		void	Append( int in_iVal, const char * in_pFormatStr = "%ld" );
		void	Append( bool in_bVal );

		#if DBGLOG_VAARGS_ENABLED
			//void	Append( const char * in_pFormatStr, ... );
			void	Append_VA( const char * in_pFormatStr, va_list in_ArgsVA );
		#else
			void	Append( const char *in_pMsg ){ Append_SimpStr( in_pMsg ); }
		#endif

		void	SetFilePath( const char *in_pFilePath );
		void	SetWriteThreshhold( int in_Count );
		void	SetWritesPerAutoFlush( int in_Count );
		void	WriteToFileNow( void );
		void	FlushNow( void );
		void	ShallowFlushNow( void );
		void	DeepFlushNow( void );

	private:
		void	ShallowFlush_PrimitiveOnly( void );
		void	DeepFlush_PrimitiveOnly( void );
	};
#endif // DBGLOG_ENABLED

#endif // DBGLOG_H
