#ifndef _ITB_MENU_EVENT_LISTENER_H_
#define _ITB_MENU_EVENT_LISTENER_H_


#include "..\..\src\BrewString.h"
#include "..\..\src\aImage.h"
#include "..\..\Module\Std\AMTPUtil.h"

#include "AEE.h"

/**
 * <p>The FP Module contains a fairly complex ui menu system along with the network 
 * interface.  The layout of this UI is dynamic and explicitly controlled by the 
 * FPModule.  Rather than build a redundant menu system, however, the FPModule 
 * instead provides this interface for feeding menu configurations into an existing
 * client menu architecture.</p>
 * 
 * <p>The Client is responsible for taking the information provided by the following
 * event handlers, then building and rendering it using the existing menu control
 * infrastructure.</p>
 * 
 * <p>The FP Module Menu System is built around the concept of menus which are serialized
 * lists of MenuEntries.</p>  
 * 
 * <p>setMenu() events will ALWAYS comprize series of event-handler calls on this interface.
 * They will be composed of the following series of calls:</p>
 * 
 * <pre>
 * setMenu(newMenuId,newMenuIndexSelected,newMenuSize)
 * setMenuEntry(newMenuId,0,entryType,...)
 * setMenuEntry(newMenuId,1,entryType,...)
 * ...
 * endSetMenu(newMenuId)
 * </pre>
 * 
 * This will occur for EVERY FPModule menu state change.</p>
 * 
 * Beyond that, setMenuEntry() can also be called at arbitrary times while a menu is currently
 * being displayed to update that menu's entries (for animations, progress reports, etc)
 * 
 * <p>There are three "conceptual classes" of menu entries:</p>
 * 
 * <p><b>Commands</b> - These are menu entries like J2ME Commands.  IF the user selects them
 * (By whatever mechanism the client determines appropriate), the client should
 * use the ITBMenuControl to pass the entryData of the selected menu entry back to the
 * FP Module and then wait for the next setMenu() or client event.</p>
 * 
 * <p><b>Inputs</b> - These are items on the Menu which do not directly control the menu,
 * but do provide a mechanism for inputting data on the menu in a manner that is 
 * user-interactive.  The UI mechanism for implementing this is at the clients discresion,
 * however, updated to the entryData and entryText should be fed back into the FPModule
 * using ITBMenuControls.menuInput().</p>
 * 
 * <p><b>Statics</b> - These are menu entries which do nothing but display static data on the screen,
 * generally requiring no user input (except for scrolling if the data does not fit).</p>
 * 
 * 
 * <p>See also: ITBMenuControls</p>
 */
class ITBMenuEventListener {
public:
	
	/**
	 * The initial event informing that a FPModule Menu State Change is taking place.
	 * 
	 * @param menuId The Unique ID of the menu being loaded.
	 * @param menuIndexSelected The Index of the menu entry initially selected.
	 * @param menuSize The number of menu entries (not including TBC.MENUENTRY_FORE and/or
	 * TBC.MENUENTRY_BACK, which are typically handled as softkeys and not menu items)
	 * @param menuName The Text Name of this Menu.  Can be null.
	 * @return true if this Menu Handler handled this particular menu Initialization,
	 * false otherwise.  If false, setMenuEntry events will NOT occur.
	 */
	virtual bool setMenu(int menuId, int menuIndexSelected, int menuSize, String menuName) = 0;
	
	/**
	 * <p>A Method which defines or updates a menuEntry on the currently defined and
	 * displayed Menu</p>
	 * 
	 * <p>A series of these events will be called after each setMenu() (and before endSetMenu())
	 * to dictate the composition of the menu during the setMenu() event series.  setMenuEntry()
	 * can also be called at any point that a For Prize Module menu is displayed to update the 
	 * data of that menu.</p>
	 * 
	 * <p>While _DYNAMIC_ menu entry types have data that can be updated, menu configuration 
	 * (entry types, number) will not change for each distinct menuId</p>
	 * 
	 * <p>The Parameters to setMenuEntry are designed to be generic enough to support a 
	 * multitude of menu Entry types, hense the specific values stored in the various
	 * parameters will depend upon the entryType parameter.  
	 * 
	 * </p>The currently supported entryTypes are:</p>
	 * 
	 * <p><b>TBC.MENUENTRY_COMMAND</b>:</p>
	 * 	A standard menu entry which provides a link to another menu (or menu
	 * 	action, it is irrellivant to the client) when the Command is selected
	 * 	by the user.  A Selected Command should be passed back to the FPModule
	 * 	via the ITBMenuControls.menuCommand(selectedCommand's entryData)
	 * <pre>String entryName:</pre> The Text To display for this Command
	 * <pre>int entryData:</pre> The Value to pass ITBMenuControls.menuCommand() if the user
	 * 	selects this entry.
	 * <pre>String entryText:</pre> null
	 * <pre>Image entryImage:</pre> null
	 * 
	 * <p><b>TBC.MENUENTRY_COMMAND_BACK</b>:</p>
	 * 	A Special subtype of Command, this is a Command that should either
	 * 	be rendered in the menu, or as a softkey in a position coherant with the
	 *  "Back" position in the phone's UI.  If selected (or softkey pressed), 
	 *  should report back to the FPModule via:
	 *  ITBMenuControl.menuCommand(TBC.MENUENTRY_COMMAND_BACK).  
	 * <pre>String entryName:</pre> The Text To display for this Command/Softkey
	 * <pre>int entryData:</pre> TBC.MENUENTRY_COMMAND_BACK
	 * <pre>String entryText:</pre> null
	 * <pre>Image entryImage:</pre> null
	 * 
	 * <p><b>TBC.MENUENTRY_COMMAND_FORE</b>:</p>
	 * 	A Special subtype of Command, this is a Command that should either
	 * 	be rendered in the menu, or as a softkey in a position coherant with the
	 *  "Forward" position in the phone's UI.  If selected (or softkey pressed), 
	 *  should report back to the FPModule via:
	 *  ITBMenuControl.menuCommand(TBC.MENUENTRY_COMMAND_FORE).  
	 * <pre>String entryName:</pre> The Text To display for this Command/Softkey
	 * <pre>int entryData:</pre> -1
	 * <pre>String entryText:</pre> TBC.MENUENTRY_COMMAND_FORE
	 * <pre>Image entryImage:</pre> null
	 * 
	 * <p><b>TBC.MENUENTRY_INPUT_TEXT</b>:</p>
	 * 	An Input Entry Class which allows the user to input text/string data
	 * 	Once input, the data should be reported to the FPModule with:
	 *  ITBMenuControl.menuInput()  
	 * <pre>String entryName:</pre> The Prompt to Use for the Text Input Box
	 * <pre>int entryData:</pre> The Maximum Length of the Text Input Box
	 * <pre>String entryText:</pre> The text to populate the input with (length < entryData)
	 * <pre>Image entryImage:</pre> null
	 * 
	 * <p><b>TBC.MENUENTRY_INPUT_SWITCH</b>:</p>
	 * 	An Input Entry Class which allows the user to make a selection based
	 * 	on a number of pre-set options.  Any change to the selection should be
	 * 	reported to the FP Module with:
	 *  ITBMenuControl.menuInput()  
	 *  The Options will never be updated after the initial setMenu() event series.
	 * <pre>String entryName:</pre> The Prompt to Use for the Switch
	 * <pre>int entryData:</pre> The Starting State of the switch
	 * <pre>String entryText:</pre> A comma Deliniated list of text to use for each option of
	 * 	the switch.
	 * <pre>Image entryImage:</pre> null
	 *
	 * <p><b>TBC.MENUENTRY_DYNAMIC_PROGRESS</b>:</p>
	 * 	A Display MenuEntry which displays a progress indication.  The
	 * 	FPModule will report further progress of this menuEntry through repeated
	 *  calls to the 'setMenu' event until entryData = 100.  
	 * <pre>String entryName:</pre> The title of the progress bar
	 * <pre>int entryData:</pre> 0-100, the event manager will continue to call the full
	 * 	setMenu() event with updated progress information until the value reaches 100
	 * <pre>String entryText:</pre> null
	 * <pre>Image entryImage:</pre> null
	 *
	 * <p><b>TBC.MENUENTRY_DYNAMIC_IMAGE</b>:</p>
	 * 	A Display MenuEntry which displays an Image.  The Image 
	 * 	will fit on the screen.  This mechanism can and will be used to animate the
	 *  For Prize Module UI.  If a particular handset cannot sufficiantly support
	 *  this animation, simply return 'false' on the update event call and
	 *  the STATIC_IMAGE menu entry updated will cease.
	 * <pre>String entryName:</pre> A Caption to display with the image, or null
	 * <pre>int entryData:</pre> The image index (if animation, 0 otherwise)
	 * <pre>String entryText:</pre> null
	 * <pre>Image entryImage:</pre> The Image to Display
	 * 
	 * <p><b>TBC.MENUENTRY_DYNAMIC_TEXT</b>:</p>
	 * 	A Display MenuEntry which displays dynamic, updating text.  This information
	 * 	would generally be displayed on a single line of the display, if the information
	 *  is wider than the display will accept, than some form of marquee should be used
	 *  to scroll the information.  This is intended to be only a single line of text.
	 * <pre>String entryName:</pre> null
	 * <pre>int entryData:</pre> -1
	 * <pre>String entryText:</pre> The Current text in the text-line
	 * <pre>Image entryImage:</pre> null
	 * 
	 * <p><b>TBC.MENUENTRY_STATIC_TEXT</b>:</p>
	 * 	A Static Display MenuEntry which displays text within the menu.
	 * 	Wrapping/Scrolling (if necessesary) is the client's responsibility.
	 *  Static Text will never be updated after the initial setMenu() event series.
	 * <pre>String entryName:</pre> null
	 * <pre>int entryData:</pre> -1
	 * <pre>String entryText:</pre> The String to display
	 * <pre>Image entryImage:</pre> null
	 *
	 * <p>These entry Types are valid as of For Prize Module version 1.0.0.  Newer clients may support new entry types
	 * however, backwards compatability is assured.  Simply return false for any entry type not explicitly stated above.</p>
	 * 
	 * <p>NOTE: Returning false could trigger the For Prize Module to call several replacement setMenuEntry events, OR an
	 * entirely new setMenu() event series.  Be prepared to handle this</p>
	 * 
	 * <p>Regardless of type, the following Parameters are used to give the entries context
	 * within the larger menu, and their values will not depend on the entryType:</p>
	 * 
	 * @param menuId The MenuId of the menu this entry is a part of (for crosschecking).
	 * @param menuIndex The Index of this Menu Entry within the current Menu.  Will be 
	 * -1 for <pre>TBC.MENUENTRY_COMMAND_FORE</pre> and <pre>TBC.MENUENTRY_COMMAND_BACK</pre>.
	 * 
	 * @return true if the client can/will handle this menu entry (render, handle interaction, input,
	 * and report back to the client), false otherwise.  See above note about returning false.
	 *
	 */
	virtual bool setMenuEntry(int menuId, int menuIndex, int entryType, String entryName, int entryData, String entryText, ConvertedBitmapInfo* entryImage) = 0;

	/**
	 * Event triggered at the end of a new menu declaration.  
	 * 
	 * In general, the other events should be used merely to aggregate the data about the the new
	 * menu layout, and then this event should actually build the new menu.  Specific Implementation
	 * is of course up to the client.
	 * 
	 * @param menuId The MenuId of the menu whose events have just finished
	 * @return true if the client can/will handle this menu, false otherwise
	 */
	virtual bool endSetMenu(int menuId) = 0;
	
};

#endif //_ITB_MENU_EVENT_LISTENER_H_
