#ifndef _TBC_H_
//#define _TBC_H_

#include "..\..\Module\Std\Debug.h"

/**
 * <p>For Prize Module System Public Constants and Configuration</p>
 * 
 * <p>This class forms the base for all communication both within the Module, and between 
 * the client and the module.  Specific Modules should extend this class for internal 
 * communication and definitions based on that particular module's functionality</p>
 * 
 * <p>Identifiers should only be scoped as explicitly public if they are to be actively
 * utilized in Client<->Module communication, otherwise, default package protected
 * scoping should be used</p>
 *
 * <p>This class is implemented as an interface.  Values can be accessed either by implementing
 * the interface and accessing via local fields, or via direct static access </p>
 * 
 * <p>The Names of these constants is garanteed for backwards compatability.  
 * <b>The values are not</b></p>
 *
 * @author E Scott Cragg. Twistbox Client Core Engineering.
 *
 * @author Tiago Teixeira. Twistbox Client Core Engineering.
 */	
	/**
	 * These are Generic NULL or EMPTY data references shared by all Modules.
	 * 
	 * They are not acccessable by clients.
	 */
	#define MENU_NULL 0
	#define MENU_OFF MENU_NULL 
	#define MODULE_STATE_OFF MENU_NULL
	
	
	/**
	 * Menu Entry Flags.
	 * 
	 * The type of a menu entry is determined by these flags in the high-order
	 * Byte of an integer.
	 * 
	 * These are not explicitly accessable by the clients, but values based
	 * based on them will be passed to the client via the menuId parameter 
	 * all methods in ITBMenuEventListener
	 */
	#define MENU_MASK_FLAGS				  0x00ff
	#define MENU_FLAG_SCREEN				  0x1000
	#define MENU_FLAG_ACTION				  0x2000
	#define MENU_FLAG_ELEMENT_LINK			  0x4000
	#define MENU_FLAG_ELEMENT_IMAGE		  0x8000
	#define MENU_FLAG_ELEMENT_PROGRESSBAR	  0xf000
	#define MENU_FLAG_ELEMENT_SWITCH		  0x0100
	#define MENU_FLAG_ELEMENT_TEXTLINE		  0x0200
	#define MENU_FLAG_ELEMENT_TEXTBOX		  0x0400
	#define MENU_FLAG_ELEMENT_TEXTINPUT	  0x0800
	
	/**
	 * Menu Actions
	 * 
	 * Menu actions are processed by the menuCommand() function of the module,
	 * They contain logic, and may or may not cause MENU_SCREEN events to
	 * take place.
	 * 
	 * Below are the default events, required by all Modules.  Module specific
	 * events will be contained in the Module's specific FPCmenu interface.
	 */
	// Roster
	#define MENU_ACTION_START 									  MENU_FLAG_ACTION
	#define MENU_ACTION_START_GAME								  MENU_ACTION_START+1
	#define MENU_ACTION_END_GAME								  MENU_ACTION_START_GAME+1
	#define MENU_ACTION_FORE									  MENU_ACTION_END_GAME+1
	#define MENU_ACTION_BACK									  MENU_ACTION_FORE+1
	#define MENU_ACTION_MODULE_START							  MENU_ACTION_BACK+1
	
	/**
	 * Module state flags
	 * 
	 * These state flags run behind the Module Notifications provided by the client controller
	 */
	// Module State flags
	#define MODULE_STATE_FLAG_MENU_ON 	0x1
	#define MODULE_STATE_FLAG_GAME_ON 	0x2
	#define MODULE_STATE_FLAG_PAUSED  		0x4
	#define MODULE_STATE_FLAG_HELP_ONLY 	0x8
	
	/* ************************** MENU ENTRIES *****************************
	 * The following events will be passed as the type parameter in
	 * ITBMenuEventListener.setMenuEntry()
	 */
	/**
	 * 	A standard menu entry which provides a link to another menu (or menu
	 * 	action, it is irrellivant to the client) when the Command is selected
	 * 	by the user.  A Selected Command should be passed back to the FPModule
	 * 	via the ITBMenuControls.menuCommand(selectedCommand's entryData)
	 */
	#define MENUENTRY_COMMAND 0
	
	/**
	 * A Special subtype of Command, this is a Command that should either
	 * be rendered in the menu, or as a softkey in a position coherant with the
	 * "Back" position in the phone's UI.  If selected (or softkey pressed), 
	 * should report back to the FPModule via:
	 * IFPMenuControl.menuCommand(TBC.MENUENTRY_COMMAND_BACK).  
	 */
	#define MENUENTRY_COMMAND_BACK MENU_ACTION_BACK
	
	/**
	 * A Special subtype of Command, this is a Command that should either
	 * be rendered in the menu, or as a softkey in a position coherant with the
	 * "Forward" position in the phone's UI.  If selected (or softkey pressed), 
	 * should report back to the FPModule via:
	 * IFPMenuControl.menuCommand(TBC.MENUENTRY_COMMAND_FORE).  
	 */
	#define MENUENTRY_COMMAND_FORE MENU_ACTION_FORE
	
	/**
	 * An Input Entry Class which allows the user to input text/string data
	 * Once input, the data should be reported to the FPModule with:
	 * IFPMenuControl.menuInput()  
	 */
	#define MENUENTRY_INPUT_TEXT MENU_FLAG_ELEMENT_TEXTINPUT
	
	/**
	 * An Input Entry Class which allows the user to make a selection based
	 * on a number of pre-set options.  Any change to the selection should be
	 * reported to the FP Module with:
	 * IFPMenuControl.menuInput()  
	 */
	#define MENUENTRY_INPUT_SWITCH MENU_FLAG_ELEMENT_SWITCH
		
	/**
	 * A Static Display MenuEntry which displays a progress indication.  The
	 * FPModule will report further progress of this menuEntry through repeated
	 * calls to the 'setMenu' event until entryData 100.  
	 */
	#define MENUENTRY_DYNAMIC_PROGRESS MENU_FLAG_ELEMENT_PROGRESSBAR
	
	/**
	 * A Display MenuEntry which displays an Image.  The Image 
	 * will fit on the screen.
	 */
	#define MENUENTRY_DYNAMIC_IMAGE MENU_FLAG_ELEMENT_IMAGE

	/**
	 * A Display MenuEntry which displays text which may be updated by the module while displayed.
	 * In general this text will be small, but some simple mechanism (either wrapping or a single 
	 * line Marquee) should be used to garantee that all text can be displayed. 
	 */
	#define MENUENTRY_DYNAMIC_TEXT MENU_FLAG_ELEMENT_TEXTLINE
	
	/**
	 * A Static Display MenuEntry which displays a bunch of text within the menu.
	 * Wrapping/Scrolling (if necessesary) is the client's responsibility  
	 */
	#define MENUENTRY_STATIC_TEXT MENU_FLAG_ELEMENT_TEXTBOX

	
	/* ************************** CLIENT_NOTIFICATIONS *****************************
	 * The following events will be passed by the client with:
	 * ITBClientEventListener.notifyClient(int code, byte[] data)
	 */
	
	/**
	 * The For Prize Module is ready to start driving the application menu.
	 * See ITBMenuEventListener for more data (Menu Listener events will occur after this event)
	 * 
	 * data will be null
	 */
	#define CLIENT_START_MENU 1
	
	/**
	 * The For Prize Module Menu has exited its internal states.  The application's
	 * built in menu should take over to drive UI.
	 * 
	 * data will be null
	 */
	#define CLIENT_END_MENU 2
	
	/**
	 * The For Prize Module is ready to start a forPrize gameplay.  Start the game.
	 * Data contains the Game's seed.
	 * 
	 * data will be null
	 */
	#define CLIENT_GAME_START 3

	/* ************************** MODULE_NOTIFICATIONS *****************************  
	 * The following events should be used by the client with: 
	 * ITBMenuControls.notifyModule(int code, int data) 
	 */
	
	/**
	 * Notify the Module that it is time to begin driving the application's user interface.
	 * Begin from the module main-menu.
	 * 
	 * data is ignored
	 */
	#define MODULE_NOTIFY_START_MAIN MODULE_STATE_FLAG_MENU_ON
	
	/**
	 * Notify the Module that it is time to begin driving the application's user interface.
	 * Show the module help tree only
	 * 
	 * data is ignored
	 */
	#define MODULE_NOTIFY_START_HELP MODULE_STATE_FLAG_MENU_ON | MODULE_STATE_FLAG_HELP_ONLY

	/**
	 * Notify the module that the application has been paused (on pauseApp Event)
	 * 
	 * data is ignored
	 */
	#define MODULE_NOTIFY_PAUSED MODULE_STATE_FLAG_PAUSED
	
	/**
	 * Notify the module that the application has been resumed (on startApp Event, can occur 
	 * on initial event) The Module will immediately start driving the user interface if it 
	 * was in control before the MODULE_NOTIFY_PAUSED event occurred
	 * 
	 * data is ignored
	 */
	#define MODULE_NOTIFY_RESUMED ~MODULE_STATE_FLAG_PAUSED
	
	/**
	 * Notify the module that gameplay has ceased and the score is valid
	 * 
	 * data is score
	 */
	#define MODULE_NOTIFY_END_GAME ~MODULE_STATE_FLAG_GAME_ON 
	
	/**
	 * Notify the module that gameplay has been cancelled and no score will be provided for
	 * this particular FP gameplay
	 * 
	 * data is ignored
	 */
	#define MODULE_NOTIFY_CANCEL_GAME ~MODULE_STATE_FLAG_GAME_ON
	
	/**
	 * Notify the module that the application is being terminated
	 * 
	 * data is ignored
	 */
	#define MODULE_NOTIFY_DESTROYED 0
	
	// Private Module Notifications
	#define MODULE_NOTIFY_START_MENU MODULE_STATE_FLAG_MENU_ON
	#define MODULE_NOTIFY_STOP_MENU ~MODULE_NOTIFY_START_MENU
	#define MODULE_NOTIFY_START_GAME MODULE_STATE_FLAG_GAME_ON
	#define MODULE_NOTIFY_END_HELP_MENU ~MODULE_NOTIFY_START_HELP
	
	/* ************************** LOCALE CONFIGURATION ********************************* */ 
	
	/**
	 * Locale: English for Continental US.
	 */
	#define LOCALE_EN_US 0
	
	/**
	 * Default ForPrize Locale
	 */
	#define LOCALE_DEFAULT LOCALE_EN_US

	#define CLIENT_GAME_END 0


#endif //_TBC_H_