/*
 * Memory Handler (c)2000 Ralf Niebecker aka Niewi
 *
 * the memory handler gives you the possibility to debug and monitor your memory allocations.
 * it takes care of all with mem_malloc(...), mem_realloc(...) or mem_strdup(...) allocated memory
 * you can always check how much memory is currently allocated and in what modul/line the allocation has been done
 *
 * call the mem_init() function at startup and the mem_cleanup() at the end of your program.
 * allocate all memory using the mem_malloc(...) or the mem_strdup(...) function 
 * free all memory using the mem_free(...) function
 */

#ifndef _MEM_H_
#define _MEM_H_

##include "AEEAppGen.h"        // Applet helper file
##include "AEEStdLib.h"        // AEE Stb Lib Services
##include "AEEGraphics.h"
##include "BrewString.h"
##include "BrewVector.h"
##include "AEEFile.h"

//to use as a callback funtion on mem alloation errors!
typedef void (*mem_errfct_t) (size_t,const char*,const char*,int);

struct MemStats
{
	int slots;		//number of valid entries in the memory table
	int slots_inuse;//currently count of slots in use
	int slots_all;	//allover used slots since startup

	int bytes;		//currently number of allocated bytes
	int bytes_all;	//allover allocated bytes since startup 
};
const int MEM_STATS_SIZE=sizeof(struct MemStats);

/************************************************************************
  Desc: initilize the Memory Handler, allocation of the Memory_Table
*************************************************************************/
struct MemStats* mem_init(int min_size=0,mem_errfct_t errfct=NULL);

/************************************************************************
 use mem_malloc, mem_realloc and mem_strdup for mem alloction instead of
 the standard c functions.  insert the compiler constants __FILE__ and
 __LINE__ for file and line, they are needed for the debug infos!
*************************************************************************/

void* mem_alloc(size_t len,
				const char* file,
				int    line);
//-----------------------------------------------------------------------------
// Desc: allocate Memory
//
// In:   size_t length       length of memory to allocate
//       char*  file         __FILE__
//       int    line         __LINE__
//
// Out:  void*               on success a pointer to the allocted memory
//                           else NULL
//-----------------------------------------------------------------------------

void* mem_realloc(void*  ptr,
				  size_t len,
				  const char* file,
				  int    line);
//-----------------------------------------------------------------------------
// Desc: reallocate Memory allocated using mem_malloc(...)
//
// In:   void*  ptr          
//       size_t length       length of memory to allocate
//       char*  file         __FILE__
//       int    line         __LINE__
//
// Out:  void*               on success a pointer to the allocted memory
//                           else NULL
//-----------------------------------------------------------------------------

char* mem_strdup(const char* text,
				const char* file,
				int   line);
//-----------------------------------------------------------------------------
// Desc: copy a String and return the copied length!
//
// In:   const char* text    a pointer to the text to duplicate
//       char*  file         __FILE__
//       int    line         __LINE__
//
// Out:  void*               on success a pointer to the allocted memory
//                           else NULL
//-----------------------------------------------------------------------------

void mem_free(void*,
			const char* info,
			const char* file,
			int line);
//-----------------------------------------------------------------------------
// Desc: free memory allocated with mem_malloc(...) or mem_strdup(...)
//
// In:   void*               the Pointer to the Memory to free
//                           we are using a reference here to be able to 
//                           set the freed pointer to NULL
//-----------------------------------------------------------------------------

void mem_stats(IFile*);
//-----------------------------------------------------------------------------
// Desc: print a Statistic
//
// In:   FILE* a file descriptor to output the statistics to
//-----------------------------------------------------------------------------

void mem_cleanup(const char*);
//-----------------------------------------------------------------------------
// Desc: Cleanup all by mem_init(...) allocated Resources
//       still allocated Memory will be listed and freed
//       you can use a NULL as Parameter to silently free all memory
//
// In:   const char*  a filename to output the list of unclosed files to
//-----------------------------------------------------------------------------

void mem_dump(const char* file);

void* mem_changeStr(void*  ptr, const char* file);
void* mem_changeStr(void*  ptr, const AECHAR* file);

#endif //_MEM_H_

